"""A collection of aws tools"""
import os
import subprocess
from os.path import abspath, dirname

from .config import cprint, get_package_dir

# docker run -it --mount src="$(pwd)",target=/app,type=bind --entrypoint=python3 $(docker build -q .)
# docker run --rm -it --entrypoint bash -p 8889:8889 --mount src="$(pwd)",target=/local_dir,type=bind 280b97f7dfc5

# cd /local_dir jupyter notebook --ip 0.0.0.0 --no-browser --allow-root --port 8889

AMAZON_LINUX_DOCKER_TAG = "pyemr/amazonlinux:latest"
AMAZON_LINUX_DOCKER_FILE = "amazonlinux.Dockerfile"

# docker run -it --mount src="$(pwd)",target=/app,type=bind --entrypoint python3 pyemr/amazonlinux:latest
# docker run -it --mount src="$(pwd)",target=/app,type=bind --entrypoint bash --cmd python pyemr/amazonlinux:latest

def launch_docker_notebook():
    """ """
    docker_build(AMAZON_LINUX_DOCKER_FILE, AMAZON_LINUX_DOCKER_TAG)
    docker_run_sh(
        AMAZON_LINUX_DOCKER_TAG, "$(pwd)", "/run_notebook.sh", it=True, p="8889:8889"
    )


def launch_docker_python(*args, **kwards):
    """

    Args:
      *args:
      **kwards:

    Returns:

    """
    args = " ".join(args)
    kwards = " ".join(["-{k} {v}" for k, v in kwards.items()])
    docker_build(AMAZON_LINUX_DOCKER_FILE, AMAZON_LINUX_DOCKER_TAG)
    docker_run_sh(
        AMAZON_LINUX_DOCKER_TAG, "$(pwd)", f"/run_python.sh {kwards} {args}", it=True
    )


def launch_docker_shell():
    """ """
    docker_build(AMAZON_LINUX_DOCKER_FILE, AMAZON_LINUX_DOCKER_TAG)
    docker_run_sh(AMAZON_LINUX_DOCKER_TAG, "$(pwd)", "", it=True)


def launch_docker_bash():
    """ """
    docker_build(AMAZON_LINUX_DOCKER_FILE, AMAZON_LINUX_DOCKER_TAG)
    docker_run_sh(AMAZON_LINUX_DOCKER_TAG, "$(pwd)", "", it=True, entry_point="bash")


def launch_pyspark():
    """ """
    docker_build(AMAZON_LINUX_DOCKER_FILE, AMAZON_LINUX_DOCKER_TAG)
    docker_run_sh(
        AMAZON_LINUX_DOCKER_TAG, "$(pwd)", "/run_pyspark.sh", it=True, p="8889:8889"
    )



def local_spark_submit(script):
    """

    Args:
      script:

    Returns:

    """
    docker_build(AMAZON_LINUX_DOCKER_FILE, AMAZON_LINUX_DOCKER_TAG)
    docker_run_sh(
        AMAZON_LINUX_DOCKER_TAG,
        "$(pwd)",
        f"/spark_submit.sh {script}",
        it=True,
        p="8889:8889",
    )


def get_project_docker_dir():
    """ """
    package_root = get_package_dir()
    docker_dir = f"{package_root}/docker/"
    return docker_dir


def is_docker_build(tag_name):
    """

    Args:
      tag_name:

    Returns:

    """
    try:
        subprocess.check_output(["docker", "inspect", "--type=image", tag_name])
        return True
    except subprocess.CalledProcessError as e:
        return False


def docker_build(dockerfile, tag_name):
    """

    Args:
      docker_dir:
      dockerfile:
      tag_name:

    Returns:

    """
    docker_dir = get_project_docker_dir()

    print(f"Building emr docker image '{dockerfile}'...")

    if is_docker_build(tag_name) == False:
        cprint(
            f"WARNING:This is the first time you using pyemr or '{dockerfile}'. It might take ~5 minutes."
        )

    build = f"docker build -t {tag_name} --file {dockerfile} ."
    os.system(f"cd {docker_dir}; {build}")


def docker_run_sh(tag_name, mount_dir, sh_cmd="", it=False, p=None, entry_point="sh"):
    """

    Args:
      tag_name:
      mount_dir:
      sh_cmd: (Default value = '')
      it: (Default value = False)
      p: (Default value = None)
      entry_point: (Default value = 'sh')

    Returns:

    """

    mount = f'src="{mount_dir}",target=/app,type=bind'
    cmd = ["docker", "run", "--mount", mount]
    if it:
        cmd.append("-it")

    if p is not None:
        cmd += ["-p", p]

    cmd += [tag_name, entry_point, sh_cmd]
    cmd = " ".join(cmd)
    print(f"Running '{cmd}': \n")
    os.system(cmd)



"""Tools for packaging poetry projects"""
import glob
import os
import os.path
import subprocess
from datetime import datetime
from os.path import abspath, dirname

# import awswrangler as wr
from subprocess import check_output

import awswrangler as wr
from loguru import logger

from .aws import get_cluster_id, get_file_s3_path, upload_to_s3_stage
from .config import (
    _get_pyproject_toml,
    add_pyemr_param,
    cprint,
    get_build_name,
    get_cluster_name,
    get_env_name,
    get_package_dir,
    get_project_name,
    get_project_type,
    get_version,
    load_config,
)
from .docker import docker_build, docker_run_sh, is_docker_build
from .emr import wait_else_cancel


def get_datetime_string():
    """Returns the datetime string based on project config"""

    conf = load_config()
    if conf["date-time"] == "now":
        return datetime.now().strftime("%Y-%m-%dT%H-%M-%S")
    if conf["date-time"] == "today":
        return datetime.now().strftime("%Y-%m-%d")
    if conf["date-time"] == "latest":
        return "latest"
    if "%" in conf["date-time"]:
        return datetime.now().strftime(conf["date-time"])

    return None


def get_local_build_path(prefix=None):
    """

    Args:
      prefix: (Default value = None)

    Returns:

    """
    env_name = get_build_name()
    if prefix is None:
        out_path = f"dist/{env_name}"
    else:
        out_path = f"dist/{env_name}"
    return out_path


def get_client_mode_runner_path():
    """ """
    package_dir = get_package_dir()
    return "{package_dir}/utils/client_mode_runner.sh"


def pack_poetry_project_in_docker():
    """Creates a pack file from the poetry project in the currently directory."""

    docker_file = "amazonlinux.Dockerfile"
    tag_name = "pyemr/amazonlinux:latest"
    conf = load_config()
    input_dir = os.getcwd()
    local_build_path = get_local_build_path("amazonlinux")

    # build the docker image
    docker_build(docker_file, tag_name)

    # run van-pack inside container
    print(f"Building package inside docker container '{docker_file}'")
    sh_cmd = f"/build.sh {local_build_path}"
    docker_run_sh(tag_name, input_dir, sh_cmd)

    return local_build_path


def upload_amazonlinux_build_s3(build_path=None):
    """

    Args:
      build_path: (Default value = None)

    Returns:

    """
    if build_path is None:
        build_path = get_local_build_path("amazonlinux")
    s3_build_path = upload_file_s3(build_path, out_dir="code")
    return s3_build_path


def get_amazonlinux_build_path_s3():
    """ """
    build_path = get_local_build_path("amazonlinux")
    s3_build_path = get_file_s3_path(build_path, "latest", "code")
    return s3_build_path


def upload_file_s3(local_path, out_dir=""):
    """

    Args:
      local_path:
      out_dir: (Default value = '')

    Returns:

    """
    date_time = get_datetime_string()
    s3_path = upload_to_s3_stage(local_path, date_time, out_dir)
    return s3_path


def most_recent(patterm):
    """

    Args:
      patterm:

    Returns:

    """
    list_of_files = glob.glob(patterm)  # * means all if need specific format then *.csv
    latest_file = max(list_of_files, key=os.path.getctime)
    return latest_file


def build(cluster_name):
    """Build the project using pack then uploads it to s3.

    Args:
      location: (Default value = "s3")
      cluster_name:

    Returns:

    """

    build_path = pack_poetry_project_in_docker()
    s3_build_path = upload_amazonlinux_build_s3(build_path)
    add_pyemr_param("latest_build", s3_build_path)
    return s3_build_path


def get_client_mode_runner_path():
    """ """
    package_root = dirname(dirname(abspath(__file__)))
    return f"{package_root}/utils/client_mode_runner.sh"


def upload_client_mode_runner_to_s3():
    """ """
    client_runner_path = get_client_mode_runner_path()
    return upload_to_s3_stage(client_runner_path, "latest", "code")


# TODO:
# finish submit command
#      test build and run. make
# update docs + howto guide
# notebook docker command

## TODO: convert from bash to python
# cluster mode

SPARK_SUBMIT_CMD_CLUSTER = """
sudo spark-submit \
--conf spark.yarn.appMasterEnv.PYSPARK_PYTHON=./env/bin/python3 \
--conf spark.yarn.dist.archives={s3_build_path}#env \
--master yarn \
--deploy-mode cluster \
{s3_script_path} \
{kwargs)
"""

SPARK_SUBMIT_CMD_CLIENT = """
sudo PYARROW_IGNORE_TIMEZONE=1 \
PYSPARK_DRIVER_PYTHON=./env/bin/python3 \
PYSPARK_PYTHON=./env/bin/python3 \
spark-submit \
--deploy-mode client \
{s3_script_path} \
{kwargs}
"""


def submit_spark_step(
    local_script_path, submit_mode, cluster_name, wait=True, **kwargs
):
    """

    Args:
      local_script_path:
      submit_mode:
      cluster_name:
      wait: (Default value = True)
      **kwargs:

    Returns:

    """

    kwargs = " ".join([f"--{k}={v}" for k, v in kwargs.items()])

    cluster_name = get_cluster_name(cluster_name)
    cluster_id = get_cluster_id(cluster_name)

    date_time = get_datetime_string()
    s3_script_path = upload_to_s3_stage(local_script_path, date_time, "code")
    script_name = local_script_path.split("/")[-1]

    s3_build_path = get_amazonlinux_build_path_s3()

    submit_mode = submit_mode.lower()

    if submit_mode == "client":
        client_runner_s3_path = upload_client_mode_runner_to_s3()
        spark_submit_cmd = SPARK_SUBMIT_CMD_CLIENT.strip().format(
            s3_script_path=s3_script_path, kwargs=kwargs
        )
        cmd = f"{client_runner_s3_path} {s3_build_path} {spark_submit_cmd}"
        script = True
    elif submit_mode == "cluster":
        spark_submit_cmd = SPARK_SUBMIT_CMD_CLUSTER.strip().format(
            s3_script_path=s3_script_path, s3_build_path=s3_build_path, kwargs=kwargs
        )
        cmd = spark_submit_cmd
        script = False
    else:
        raise ValueError(f"No spark submit mode called '{submit_mode}'.")

    env_name = get_env_name()
    step_name = f"{env_name}:spark-submit:{local_script_path}"

    step_id = wr.emr.submit_step(
        cluster_id=cluster_id,
        name=step_name,
        command=cmd,
        script=script,
    )

    print("")
    cprint(
        f"Running '{local_script_path}' on EMR cluster '{cluster_name}' in enviroment '{env_name}'"
    )

    if wait:
        wait_else_cancel(
            cluster_id,
            step_id,
        )

    return step_id



"""A set of tools for manipulating project toml files"""
import os
import os.path
import subprocess
from os.path import abspath, dirname

# import awswrangler as wr
from typing import Dict

import tomlkit

REQUIRED_PARAM = ["s3-staging-dir", "cluster-name", "stage", "region"]

DEFAULT_CONFIG_TEMPLATE = {
    "cluster-name": "EDA",
    "s3-staging-dir": "s3://<bucket-name>/<your-staging-directory>",
    "stage": "dev",
    "date-time": "%Y-%m-%dT%H-%M-%S",
    "region": "eu-west-1",
}

DEFAULT_DEPENDENCIES = dict(
    Cython="0.29.24",
    pybind11="1.0.0",
    pythran="0.10.0",
    scipy="^1.2.0",
    Pillow="6.2.0",
    koalas="^1.8.2",
)

GIT_IGNORE_PATTERNS = [
    ".DS_Store",
    "**/.DS_Store",
    ".docker_venv",
    "**/docker_venv",
]

SIMPLE_KOALAS_EXAMPLE = """
import pandas as pd
import numpy as np
import databricks.koalas as ks
from pyspark.sql import SparkSession

kdf = ks.DataFrame(
{'a': [1, 2, 3, 4, 5, 6],
 'b': [100, 200, 300, 400, 500, 600],
 'c': ["one", "two", "three", "four", "five", "six"]},
index=[10, 20, 30, 40, 50, 60])

print(kdf.head())
""".strip()


def get_package_dir():
    """ """
    return dirname(dirname(abspath(__file__)))


def append_git_ignore(git_ignore_path=".gitignore"):
    """

    Args:
      git_ignore_path: (Default value = '.gitignore')

    Returns:

    """

    with open(git_ignore_path, "r") as f:
        git_ignore = f.read()

    git_ignore = git_ignore.split("\n")
    for pattern in GIT_IGNORE_PATTERNS:
        if pattern not in git_ignore:
            git_ignore.append(pattern)

    with open(git_ignore_path, "w") as f:
        f.write("\n".join(git_ignore))


def create_git_ignore(git_ignore_path=".gitignore"):
    """

    Args:
      git_ignore_path: (Default value = '.gitignore')

    Returns:

    """
    if not os.path.isfile(git_ignore_path):
        with open(git_ignore_path, "w") as f:
            f.write("\n".join(GIT_IGNORE_PATTERNS))
    else:
        append_git_ignore(git_ignore_path)


def get_cluster_name(cluster_name):
    """

    Args:
      cluster_name:

    Returns:

    """
    emr_conf = load_config()
    if cluster_name in ["", None, "None"]:
        cluster_name = emr_conf["cluster-name"]
    return cluster_name


def get_version():
    """ """
    config = _get_pyproject_toml()
    return config["tool"]["poetry"]["version"]


def get_s3_staging_dir():
    """ """
    emr_conf = load_config()
    s3_stage_dir = emr_conf["s3-staging-dir"]
    if s3_stage_dir.endswith("/"):
        s3_stage_dir = s3_stage_dir[:-1]
    return s3_stage_dir


def get_project_name():
    """ """
    proj_config = _get_pyproject_toml()
    project_name = proj_config["tool"]["poetry"]["name"]
    return project_name


def get_env_name():
    """ """
    conf = load_config()
    project_name = get_project_name()
    env_name = project_name.replace(" ", "_").lower()
    version = str(get_version()).replace(".", "_").lower()
    version = str(get_version()).replace(".", "_").lower()
    return f"{env_name}_{conf['stage']}_{version}"


def get_build_name():
    """Get the project name"""
    project_name = get_env_name()
    return f"{project_name}.tar.gz"


def get_project_type():
    """Get the type of project. e.g. poetry."""
    if os.path.isfile("./pyproject.toml"):
        return "poetry"

    raise ValueError('No "./pyproject.toml" found.')


def _pyproject_toml_exists(pyproj_path: str = "./pyproject.toml"):
    """

    Args:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")

    Returns:

    """
    return os.path.isfile(pyproj_path)


def _get_pyproject_toml(pyproj_path: str = "./pyproject.toml") -> Dict[str, str]:
    """Loads the pyproject toml file as a dict.

    Args:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")
      pyproj_path: str:  (Default value = "./pyproject.toml")

    Returns:

    """

    if _pyproject_toml_exists(pyproj_path):
        with open(pyproj_path, "r", encoding="utf8") as pyproject:
            file_contents = pyproject.read()
        return tomlkit.parse(file_contents)
    else:
        raise ValueError(f"'{pyproj_path}' does not exists. Run 'pyemr init'.")

    return {}


def _write_pyproject_toml(file_contents: dict, pyproj_path: str = "./pyproject.toml"):
    """Writes a dict into a toml file

    Args:
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")
      file_contents: dict:
      pyproj_path: str:  (Default value = "./pyproject.toml")

    Returns:

    """
    with open(pyproj_path, "w", encoding="utf8") as pyproject:
        pyproject.write(tomlkit.dumps(file_contents))


def load_config():
    """Loads the config parameters from the project toml file."""
    tml = _get_pyproject_toml()
    return tml["tool"]["pyemr"]


def _update_git_ignore():
    """ """
    pass


def set_param(key, value):
    """

    Args:
      key:
      value:

    Returns:

    """

    tml = _get_pyproject_toml()

    if "tool" not in tml:
        tml["tool"] = {}

    if "pyemr" not in tml["tool"]:
        tml["tool"]["pyemr"] = {}

    tml[key] = value
    _write_pyproject_toml(tml)


STDOUT_COLORS = dict(
    HEADER="\033[95m",
    OKBLUE="\033[94m",
    OKCYAN="\033[96m",
    OKGREEN="\033[92m",
    WARNING="\033[93m",
    FAIL="\033[91m",
    ENDC="\033[0m",
    BOLD="\033[1m",
    UNDERLINE="\033[4m",
)


def color_text(text, style="OKCYAN"):
    """

    Args:
      text:
      style: (Default value = 'OKCYAN')

    Returns:

    """
    return f"{STDOUT_COLORS[style]}{text}{STDOUT_COLORS['ENDC']}"


def cinput(variable_name, default):
    """

    Args:
      variable_name:
      default:

    Returns:

    """
    prompt = color_text(variable_name, "OKCYAN")
    prompt += color_text(" [", "OKCYAN")
    prompt += color_text(str(default), "OKGREEN")
    prompt += color_text("]", "OKCYAN")

    print(prompt, end="")
    value = input() or default
    print(value)
    return value


def cprint(text, color="OKCYAN"):
    """

    Args:
      text:
      color: (Default value = 'OKCYAN')

    Returns:

    """
    return print(color_text(text, color))


def init_pyemr_param(cluster_name, s3_stage_dir, stage, region):
    """Append EMR pack aparameters to the project toml

    Args:
      cluster_name:
      stage_dir:
      s3_stage_dir:
      stage:
      region:

    Returns:

    """

    tml = _get_pyproject_toml()

    if "tool" not in tml:
        tml["tool"] = {}

    if "pyemr" not in tml["tool"]:
        tml["tool"]["pyemr"] = {}

    if cluster_name:
        tml["tool"]["pyemr"]["cluster-name"] = cluster_name

    if s3_stage_dir:
        tml["tool"]["pyemr"]["s3-staging-dir"] = s3_stage_dir

    if stage:
        tml["tool"]["pyemr"]["stage"] = stage

    if region:
        tml["tool"]["pyemr"]["region"] = region

    for variable_name, default_value in DEFAULT_CONFIG_TEMPLATE.items():
        if variable_name not in tml["tool"]["pyemr"]:
            if variable_name in REQUIRED_PARAM:
                value = cinput(variable_name, default_value)
            else:
                value = default_value
            tml["tool"]["pyemr"][variable_name] = value

    print("[tool.pyemr]")
    print(tomlkit.dumps(tml["tool"]["pyemr"]))

    _write_pyproject_toml(tml)
    return True


def add_dependencies(dep):
    """

    Args:
      dep:

    Returns:

    """
    tml = _get_pyproject_toml()
    tml["tool"]["poetry"]["dependencies"].update(dep)
    _write_pyproject_toml(tml)


def add_packages_to_toml(package_path):
    """

    Args:
      package_path:

    Returns:

    """

    package_obj = tomlkit.inline_table()
    tml = _get_pyproject_toml()
    package_obj["include"] = package_path
    if "packages" not in tml["tool"]["poetry"]:
        tml["tool"]["poetry"]["packages"] = []

    tml["tool"]["poetry"]["packages"].append(package_obj)
    _write_pyproject_toml(tml)


def add_pyemr_param(key, value):
    """

    Args:
      key:
      value:

    Returns:

    """
    tml = _get_pyproject_toml()

    if "tool" not in tml:
        tml["tool"] = {}

    if "pyemr" not in tml["tool"]:
        tml["tool"]["pyemr"] = {}

    tml["tool"]["pyemr"][key] = value
    _write_pyproject_toml(tml)


def create_koalas_example(out_path):
    """

    Args:
      out_path:

    Returns:

    """
    with open(f"{out_path}/script.py", "w") as f:
        f.write(SIMPLE_KOALAS_EXAMPLE.strip())


def add_main_package():
    """ """
    tml = _get_pyproject_toml()
    name = tml["tool"]["poetry"]["name"]
    package_path = name.replace(" ", "_").lower()
    os.system(f"mkdir {package_path}")
    create_koalas_example(package_path)
    add_packages_to_toml(package_path)


def add_scr_package():
    """ """
    package_path = "src"
    os.system(f"mkdir {package_path}")
    create_koalas_example(package_path)
    add_packages_to_toml(package_path)


def init_pyemr(project_name, cluster_name, s3_stage_dir, stage, region):
    """

    Args:
      cluster_name:
      stage_dir:
      project_name:
      s3_stage_dir:
      stage:
      region:

    Returns:

    """

    dep = [
        f"--dependency={name}={version}"
        for name, version in DEFAULT_DEPENDENCIES.items()
    ]

    if not _pyproject_toml_exists():
        if project_name is None or project_name == "":
            project_name = cinput("Project Name", "tmp")
        args = [
            "poetry",
            "init",
            "--python=>=3.7,<3.10",
            "-n",
            "--name",
            project_name,
            "--quiet",
        ] + dep
        proc = subprocess.Popen(args)
        proc.communicate()

    # install default dependencies
    # add_dependencies(DEFAULT_DEPENDENCIES)
    add_scr_package()
    init_pyemr_param(cluster_name, s3_stage_dir, stage, region)
    create_git_ignore()
    os.system("open pyproject.toml")


"""
    --project_name=PROJECT_NAME
        Default: ''
    --cluster_name=CLUSTER_NAME
        Default: ''
        (Default value = '')
    --s3_stage_dir=S3_STAGE_DIR
        Default: ''
    --stage=STAGE
        Default: ''
    --region=REGION
        Default: ''
"""



# pylint: disable=R0201
"""Command Line Interface"""

import os
from pprint import pprint

# import awswrangler as wr
from subprocess import check_output

import fire

from .utils.aws import (
    cancel_step,
    describe_cluster,
    describe_step,
    download_erm_step_logs,
    get_step_state,
    list_steps,
    print_emr_log_files_lines,
    ssm_cluster,
)
from .utils.build import (
    build,
    pack_poetry_project_in_docker,
    submit_spark_step,
    upload_amazonlinux_build_s3,
    upload_file_s3,
)
from .utils.config import add_pyemr_param, init_pyemr
from .utils.docker import (
    launch_docker_bash,
    launch_docker_notebook,
    launch_docker_python,
    launch_docker_shell,
    launch_pyspark,
    local_spark_submit,
)
from .utils.emr import get_clusters_list_df


class Cli:
    """Command line interface for the package."""

    def init(
        self, project_name="", target_cluster="", s3_stage_dir="", stage="", region=""
    ):
        """Create a pyproject.toml containing pyemr config.

        Args:
          cluster_name: (Default value = '')
          stage_dir: (Default value = '')
          project_name: (Default value = '')
          target_cluster: (Default value = "")
          s3_stage_dir: (Default value = "")
          stage: (Default value = '')
          region: (Default value = '')

        Returns:

        """
        init_pyemr(project_name, target_cluster, s3_stage_dir, stage, region)

    def ssh(self, cluster_name=""):
        """A proxy name for ssm.

        Args:
          cluster_name: (Default value = "")

        Returns:

        """
        self.ssm(cluster_name)

    def ssm(self, cluster_name=""):
        """smm into the cluster master node.

        Args:
          cluster_name: (Default value = "")

        Returns:

        """
        ssm_cluster(cluster_name)

    def ssmbash(self, cluster_name, cmd):
        """run a command on a cluster master node

        Args:
          cluster_name:
          cmd:

        Returns:

        """
        # ssm_bash(cluster_name, cmd)

    def build(self, cluster_name=""):
        """Zips the python package and dependencies and then uploads them s3 staging directory.

        Args:
          cluster_name: (Default value = "")

        Returns:

        """
        build(cluster_name)

    def install(self, cluster_name=""):
        """

        Args:
          cluster_name: (Default value = "")

        Returns:

        """
        self.build(cluster_name)

    def submit(
        self, script, submit_mode="client", cluster_name="", wait=True, **kwargs
    ):
        """

        Args:
          script:
          submit_mode: (Default value = 'client')
          cluster_name: (Default value = '')
          wait: (Default value = True)
          *args:
          **kwargs:

        Returns:

        """
        submit_spark_step(script, submit_mode, cluster_name, wait, **kwargs)

    def config(self, toml_path="pyproject.toml"):
        """

        Args:
          toml_path: (Default value = "./pyproject.toml")

        Returns:

        """
        input_dir = os.getcwd()
        check_output(["open", f"{input_dir}/{toml_path}"])

    def docker_build(self):
        """ """
        pack_poetry_project_in_docker()

    def upload_build(self):
        """ """
        print(upload_amazonlinux_build_s3())

    def bsubmit(self, location="s3"):
        """

        Args:
          location: (Default value = "s3")

        Returns:

        """
        self.buildsubmit(location)

    def notebook():
        """Starts a spark linux notebook in current directory"""

    def pyspark(self):
        """Starts a pyspark session in doker with the current director booted."""

    def export(self, type):
        """

        Args:
          type:

        Returns:

        """
        assert type in ["aws", "bash", "python", "airflow", "awswrangler"]
    

    def s3(self, cmd):
        """

        Args:
          cmd:

        Returns:

        """

    def list_steps(
        self, cluster_name="", n=10, step_name="{env_name}:*", states="*", all=False
    ):
        """

        Args:
          cluster_name: (Default value = "")
          n: (Default value = 10)
          step_name: (Default value = "*{env_name}*")
          states: (Default value = '*')
          all: (Default value = False)

        Returns:

        """
        if all:
            step_name = states = "*"
        pprint(list_steps(cluster_name, n, step_name, states))

    def cancel_step(
        self, step_id="", cluster_name="", step_name_pattern="{env_name}:*", state="*"
    ):
        """

        Args:
          step_id: (Default value = "")
          cluster_name: (Default value = "")
          step_name_pattern: (Default value = "*{env_name}*")
          state: (Default value = "*")

        Returns:

        """
        pprint(cancel_step(cluster_name, step_id, step_name_pattern, state))

    def describe_step(
        self, step_id="", cluster_name="", name="{env_name}:*", state="*"
    ):
        """

        Args:
          step_id: (Default value = "")
          cluster_name: (Default value = "")
          name: (Default value = "*{env_name}*")
          state: (Default value = "*")

        Returns:

        """
        pprint(describe_step(cluster_name, step_id, name, state))

    def download_logs(
        self,
        out_dir="./logs",
        step_id="",
        cluster_name="",
        name="*{env_name}*",
        state="*",
    ):
        """

        Args:
          step_id: (Default value = "")
          out_dir: (Default value = "./logs")
          cluster_name: (Default value = "")
          name: (Default value = "*{env_name}*")
          state: (Default value = "*")

        Returns:

        """
        download_erm_step_logs(cluster_name, step_id, name, state, out_dir=out_dir)

    def logs(
        self,
        log_type,
        step_id="",
        n=30,
        cluster_name="",
        name="*{env_name}*",
        state="*",
        out_dir="./logs",
    ):
        """

        Args:
          log_type:
          step_id: (Default value = "")
          n: (Default value = 30)
          cluster_name: (Default value = "")
          name: (Default value = "*{env_name}*")
          state: (Default value = "*")
          out_dir: (Default value = "./logs")

        Returns:

        """
        print_emr_log_files_lines(
            log_type, n, step_id, cluster_name, name, state, out_dir
        )
        # shutil.rmtree(out_dir)

    def stderr(
        self,
        step_id="",
        n=30,
        cluster_name="",
        name="*{env_name}*",
        state="*",
        out_dir="./logs",
    ):
        """

        Args:
          step_id: (Default value = "")
          n: (Default value = 30)
          cluster_name: (Default value = "")
          name: (Default value = "*{env_name}*")
          state: (Default value = "*")
          out_dir: (Default value = "./logs")

        Returns:

        """
        self.logs("stderr", step_id, n, cluster_name, name, state, out_dir)

    def stdout(
        self,
        step_id="",
        n=30,
        cluster_name="",
        name="*{env_name}*",
        state="*",
        out_dir="./logs",
    ):
        """

        Args:
          step_id: (Default value = "")
          n: (Default value = 30)
          cluster_name: (Default value = "")
          name: (Default value = "*{env_name}*")
          state: (Default value = "*")
          out_dir: (Default value = "./logs")

        Returns:

        """
        self.logs("stdout", step_id, n, cluster_name, name, state, out_dir)

    def state(self, step_id="", cluster_name="", name="*{env_name}*", state="*"):
        """

        Args:
          step_id: (Default value = "")
          cluster_name: (Default value = "")
          name: (Default value = "*{env_name}*")
          state: (Default value = "*")

        Returns:

        """
        get_step_state(cluster_name, step_id, name, state)

    def describe_cluster(self, cluster_name=""):
        """

        Args:
          cluster_name: (Default value = "")

        Returns:

        """
        pprint(describe_cluster(cluster_name))

    def list_clusters(self, states=["RUNNING", "WAITING"], n=10):
        """

        Args:
          states: (Default value = ['RUNNING')
          'WAITING']:
          n: (Default value = 10)
          "WAITING"]:

        Returns:

        """
        print(get_clusters_list_df(states, n))

    def ls(
        self,
        entity_type,
        step_name_pattern="*{env_name}*",
        states=["RUNNING", "WAITING"],
        n=10,
        *args,
        **kwards,
    ):
        """

        Args:
          entity_type:
          step_name_pattern: (Default value = "*{env_name}*")
          states: (Default value = ['RUNNING')
          'WAITING']:
          n: (Default value = 10)
          *args:
          **kwards:
          "WAITING"]:

        Returns:

        """

        if entity_type in "clusters":
            self.list_clusters(*args, **kwards)
        if entity_type in "steps":
            self.list_steps(*args, **kwards)

    def s3_cp(self, local_path, out_dir="."):
        """

        Args:
          local_path:
          out_dir: (Default value = '.')

        Returns:

        """
        print(upload_file_s3(local_path, out_dir))

    def set(self, key, value):
        """

        Args:
          key:
          value:

        Returns:

        """
        add_pyemr_param(key, value)

    def notebook(self):
        """ """
        launch_docker_notebook()

    def python(self, *args, **kwards):
        """

        Args:
          *args:
          **kwards:

        Returns:

        """
        launch_docker_python(*args, **kwards)

    def sh(self):
        """ """
        launch_docker_shell()

    def bash(self):
        """ """
        launch_docker_bash()

    def pyspark(self):
        """ """
        launch_pyspark()

    def test(self, script):
        """

        Args:
          script:

        Returns:

        """
        local_spark_submit(script)

    def export(self, *args):
        """

        Args:
          *args:

        Returns:

        """
        raise ValueError("This features is not available yet.")


def main():
    """ """
    fire.Fire(Cli)


if __name__ == "__main__":
    main()



"""A collection of aws tools"""
import time

import awswrangler as wr
import boto3
import pandas as pd
from loguru import logger
from tqdm import tqdm


def cancel_emr_step(cluster_id, step_id):
    """

    Args:
      cluster_id:
      step_id:

    Returns:

    """
    client = boto3.client("emr")
    response = client.cancel_steps(
        ClusterId=cluster_id,
        StepIds=[step_id],
    )
    logger.info(f"Canceled step {step_id}")
    return response


def yield_step_state(cluster_id, step_id, sleep=4):
    """

    Args:
      cluster_id:
      step_id:
      sleep: (Default value = 4)

    Returns:

    """

    while True:
        time.sleep(sleep)
        state = wr.emr.get_step_state(cluster_id, step_id)
        yield state


def wait_else_cancel(cluster_id, step_id, desc="Waiting"):
    """

    Args:
      cluster_id:
      step_id:
      desc: (Default value = 'Waiting')

    Returns:

    """

    last_state = None
    pbar = tqdm(
        yield_step_state(cluster_id, step_id, sleep=5),
        desc=f"{desc}:step_id={step_id},state=SUBMITTING",
    )

    # wait for step to complete
    try:
        for state in pbar:
            if state != last_state:
                pbar.set_description(f"{desc}:step_id={step_id},state={state}")

            last_state = state
            if state in ["COMPLETED", "FAILED"]:
                break
    except KeyboardInterrupt:
        cancel_emr_step(cluster_id, step_id)


def get_clusters_list_df(states, n) -> pd.DataFrame:
    """Returns a dictionary of cluster and cluster ids

    Args:
      states:
      n:

    Returns:

    """

    res = {}
    client = boto3.client("emr")
    clusters = client.list_clusters(ClusterStates=states)
    clusters = pd.json_normalize(clusters["Clusters"], sep="_").sort_values(
        "Status_Timeline_CreationDateTime", ascending=False
    )

    return clusters[["Id", "Name", "Status_State", "Status_Timeline_CreationDateTime"]]



"""A collection of aws tools"""
import fnmatch
import gzip
import os
import os.path
import pathlib
from os.path import abspath, dirname

import boto3
import pandas as pd
from loguru import logger
from tqdm import tqdm

from .config import (
    _get_pyproject_toml,
    cprint,
    get_cluster_name,
    get_env_name,
    get_s3_staging_dir,
    get_version,
    load_config,
)
from .emr import cancel_emr_step

# import awswrangler as wr


CLUSTER_STATES = ["WAITING", "RUNNING"]


def _upload_to_s3(
    local_path,
    s3_path,
):
    """Upload file to s3

    Args:
      local_path:
      s3_path:

    Returns:

    """
    _, _, bucket, key = s3_path.split("/", 3)
    s3 = boto3.resource("s3")  # pylint: disable=C0103

    print("")
    cprint(f"Uploading '{local_path}' to s3...")
    print(f"out_path={s3_path}")

    file_size = os.stat(local_path).st_size

    def callback(bytes_transferred):
        """

        Args:
          bytes_transferred:

        Returns:

        """
        return pbar.update(bytes_transferred)

    with tqdm(
        total=file_size, unit="B", unit_scale=True, desc=f"Uploading '{local_path}'"
    ) as pbar:
        s3.meta.client.upload_file(
            Filename=local_path,
            Bucket=bucket,
            Key=key,
            Callback=callback,
        )

    print(f"Uploaded '{local_path}' to '{s3_path}'")


def copy_on_s3(path, new_path):
    """Copy a blob on s3 to another s3 location

    Args:
      path:
      new_path:

    Returns:

    """
    print(f"Copying '{path}' to '{new_path}'.")
    s3 = boto3.resource("s3")  # pylint: disable=C0103
    _, _, bucket, key = path.split("/", 3)
    _, _, new_bucket, new_key = new_path.split("/", 3)

    copy_source = {"Bucket": bucket, "Key": key}

    bucket = s3.Bucket(new_bucket)
    bucket.copy(copy_source, new_key)


def get_staging_dir():
    """Returns the staging directory along with version and stage."""
    conf = load_config()
    print(conf)
    stage = conf["stage"]
    version = get_version()
    s3_stage_dir = get_s3_staging_dir()
    stage_path = f"{s3_stage_dir}/stage={stage}/version={version}"
    return stage_path


def get_clusters() -> dict:
    """Returns a dictionary of cluster and cluster ids"""

    res = {}
    client = boto3.client("emr")
    clusters = client.list_clusters(ClusterStates=CLUSTER_STATES)
    for cluster in clusters["Clusters"]:
        res[cluster["Name"]] = cluster["Id"]

    return res


def get_cluster_id(cluster_name: str) -> str:
    """

    Args:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:

    Returns:

    """

    if (cluster_name is None) or (cluster_name == "") or not cluster_name:
        cluster_name = load_config()["cluster-name"]

    client = boto3.client("emr")
    clusters = client.list_clusters(ClusterStates=CLUSTER_STATES)
    for cluster in clusters["Clusters"]:
        if cluster["Name"] == cluster_name:
            return cluster["Id"]

    for cluster in clusters["Clusters"]:
        if (type(cluster["Name"]) == str) and (type(cluster_name) == str):
            if cluster["Name"].lower() == cluster_name.lower():
                return cluster["Id"]

    raise ValueError(
        f"No cluster called '{cluster_name}' in states '{CLUSTER_STATES}' ."
    )


def get_file_s3_path(local_file_path, datetime, suffix):
    """

    Args:
      file_name:
      datetime: (Default value = "latest")
      suffix: (Default value = "code")
      local_file_path:

    Returns:

    """
    file_name = local_file_path.split("/")[-1]
    staging_dir = get_staging_dir()
    if datetime == "latest":
        return f"{staging_dir}/{suffix}/latest/{file_name}"

    return f"{staging_dir}/{suffix}/datetime={datetime}/{file_name}"


def upload_to_s3_stage(in_path, datetime, suffix):
    """

    Args:
      in_path:
      datetime: (Default value = "latest")
      suffix: (Default value = "code")

    Returns:

    """

    s3_path = get_file_s3_path(in_path, datetime, suffix)
    _upload_to_s3(in_path, s3_path)

    if datetime != "latest":
        s3_path_latest = get_file_s3_path(in_path, "latest", suffix)
        copy_on_s3(s3_path, s3_path_latest)
        return s3_path_latest

    return s3_path


def describe_cluster(cluster_name):
    """

    Args:
      cluster_name:

    Returns:

    """
    client = boto3.client("emr")
    cluster_id = get_cluster_id(cluster_name)
    cluster_desc = client.describe_cluster(ClusterId=cluster_id)
    return cluster_desc


def get_log_url(cluster_name):
    """

    Args:
      cluster_name:

    Returns:

    """
    desc = describe_cluster(cluster_name)
    return desc["Cluster"]["LogUri"]


def get_master_ec2_instance_id(cluster_name):
    """Returns the master node ec2 instanceid.

    Args:
      cluster_name:

    Returns:

    """

    client = boto3.client("emr")
    cluster_id = get_cluster_id(cluster_name)
    cluster_desc = client.describe_cluster(ClusterId=cluster_id)
    master_public_dns_name = cluster_desc["Cluster"]["MasterPublicDnsName"]

    instance_info = client.list_instances(ClusterId=cluster_id)
    for instance in instance_info["Instances"]:
        if master_public_dns_name == instance["PublicDnsName"]:
            master_ec2_instanceid = instance["Ec2InstanceId"]
            print(
                f"'{cluster_name}' cluster master ec2 instance id = {master_ec2_instanceid}"
            )
            return master_ec2_instanceid

    raise ValueError("No ec2 instance id found for {cluster_name}.")


def ssm_cluster(cluster_name: str):
    """Opens an ssm session into the master node of the given cluster

    Args:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:
      cluster_name: str:

    Returns:

    """
    master_ec2_instanceid = get_master_ec2_instance_id(cluster_name)
    print(f'Running "aws ssm start-session --target {master_ec2_instanceid}"')
    os.system(f"aws ssm start-session --target {master_ec2_instanceid}")


def ssm_bash(cmd):
    """

    Args:
      cmd:

    Returns:

    """

    emr_conf = load_config()
    instance_id = get_cluster_id(emr_conf["cluster-name"])

    ssm_client = boto3.client("ssm")
    response = ssm_client.send_command(
        InstanceIds=[instance_id],
        DocumentName="AWS-RunShellScript",
        Parameters={"commands": cmd},
    )

    return response


def cancel_step(cluster_name, step_id, step_name_pattern, state_pattern):
    """

    Args:
      cluster_name:
      step_id:
      step_name_pattern:
      state_pattern:

    Returns:

    """

    if step_id is None or step_id == "":
        step_id = get_last_submitted_step(
            cluster_name, step_name_pattern, state_pattern
        )

    cluster_id = get_cluster_id(cluster_name)
    boto3.client("emr")
    response = cancel_emr_step(cluster_id, step_id)

    return response


def list_steps(cluster_name, n, step_name_pattern, state_patterm):
    """List steps

    Args:
      cluster_name:
      n:
      step_name_pattern:
      state_patterm:

    Returns:

    """

    cluster_name = get_cluster_name(cluster_name)
    step_name_pattern = str(step_name_pattern)

    if "{env_name}" in step_name_pattern:
        step_name_pattern = step_name_pattern.format(env_name=get_env_name())

    logger.info(
        f"Listing steps on '{cluster_name}' matching name '{step_name_pattern}' in state '{state_patterm}'."
    )

    cluster_id = get_cluster_id(cluster_name)
    client = boto3.client("emr")
    response = client.list_steps(ClusterId=cluster_id)

    steps = pd.json_normalize(response["Steps"], sep="_").sort_values(
        "Status_Timeline_CreationDateTime", ascending=False
    )

    if step_name_pattern not in ["", "*", "None", "True"]:
        mateched_names = fnmatch.filter(steps.Name.tolist(), step_name_pattern)
        steps = steps[steps.Name.isin(mateched_names)]

    if state_patterm not in ["", "*", "None", None]:
        mateched_status_state = fnmatch.filter(
            steps.Status_State.astype(str).tolist(), str(state_patterm)
        )
        steps = steps[steps.Status_State.isin(mateched_status_state)]

    if n is not None or n != "":
        steps = steps.head(n)

    steps = steps[["Id", "Name", "Status_State", "Status_Timeline_CreationDateTime"]]
    return steps


def get_last_submitted_step(cluster_name, name_pattern, state):
    """

    Args:
      cluster_name:
      name_pattern:
      state:

    Returns:

    """

    steps = list_steps(cluster_name, None, name_pattern, state)

    if len(steps) > 0:
        step_id = steps["Id"].tolist()[0]
        print(f"Last submitted step '{step_id}'")
        return step_id

    if "{env_name}" in name_pattern:
        name_pattern = name_pattern.format(env_name=get_env_name())

    raise ValueError(
        f"No running steps for cluster_name='{cluster_name}' and name_pattern='{name_pattern}'"
    )


def describe_step(cluster_name, step_id, name_pattern, state):
    """

    Args:
      cluster_name:
      step_id:
      name_pattern:
      state:

    Returns:

    """

    if step_id is None or step_id == "":
        step_id = get_last_submitted_step(cluster_name, name_pattern, state)

    cluster_id = get_cluster_id(cluster_name)
    client = boto3.client("emr")
    response = client.describe_step(
        ClusterId=cluster_id,
        StepId=step_id,
    )
    return response


def get_failed_s3_log_file_path(cluster_name, step_id, name_pattern, state):
    """

    Args:
      cluster_name:
      step_id:
      name_pattern:
      state:

    Returns:

    """

    log_url = get_log_url(cluster_name)
    cluster_id = get_cluster_id(cluster_name)
    log_url = f"{log_url}{cluster_id}/steps/{step_id}"
    return log_url


import botocore


def download_s3_file(s3_path, out_dir):
    """

    Args:
      s3_path:
      out_dir:

    Returns:

    """

    _, _, bucket, key = s3_path.split("/", 3)
    file_name = key.split("/")[-1]

    s3_client = boto3.client("s3")

    pathlib.Path(out_dir).mkdir(parents=True, exist_ok=True)
    print(f"Downloading 's3://{bucket}/{key}' to '{out_dir}/{file_name}'")
    try:
        s3_client.download_file(bucket, key, f"{out_dir}/{file_name}")
    except botocore.exceptions.ClientError as e:
        if e.response["Error"]["Code"] == "404":
            raise ValueError(f"Object '{s3_path}' does not exists.")
        raise


def download_erm_step_log_gz(
    cluster_name, step_id, name_pattern, state, out_dir, log_type
):
    """

    Args:
      cluster_name:
      step_id:
      name_pattern:
      state:
      out_dir:
      log_type:

    Returns:

    """

    if step_id is None or step_id == "":
        step_id = get_last_submitted_step(cluster_name, name_pattern, state)

    s3_path = get_failed_s3_log_file_path(cluster_name, step_id, name_pattern, state)
    out_dir = f"{out_dir}/{step_id}"
    download_s3_file(f"{s3_path}/{log_type}.gz", out_dir)
    return f"{out_dir}/{log_type}.gz"


def download_erm_step_logs(cluster_name, step_id, name_pattern, state, out_dir):
    """

    Args:
      cluster_name:
      step_id:
      name_pattern:
      state:
      out_dir:

    Returns:

    """
    for log_type in ["stderr", "stdout", "syslog", "controller"]:
        download_erm_step_log_gz(
            cluster_name, step_id, name_pattern, state, out_dir, log_type
        )


def print_tail_gzip_files(in_path, n):
    """

    Args:
      in_path:
      n:

    Returns:

    """
    print(f"> tail -n {n} '{in_path}': ")
    print("\n")
    lines = []
    with gzip.open(in_path, "r") as fin:
        for line in fin:
            lines.append(line.decode("utf-8")[:-1])

    lines = lines[-n:]
    for line in lines:
        print(line)
    print("\n")


def get_step_state(cluster_name, step_id, name_pattern, state):
    """

    Args:
      cluster_name:
      step_id:
      name_pattern:
      state:

    Returns:

    """
    desc = describe_step(cluster_name, step_id, name_pattern, "*")["Step"]
    print(f"Step {desc['Name']} is in state {desc['Status']['State']}.")


def print_emr_log_files_lines(
    log_type, n, step_id, cluster_name, name_pattern, state, out_dir
):
    """

    Args:
      log_type:
      n:
      step_id:
      cluster_name:
      name_pattern:
      state:
      out_dir:

    Returns:

    """
    local_log_path = download_erm_step_log_gz(
        cluster_name, step_id, name_pattern, state, out_dir, log_type
    )
    print_tail_gzip_files(local_log_path, n)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyemr', 'pyemr.pyemr', 'pyemr.pyemr.utils', 'pyemr.utils']

package_data = \
{'': ['*'],
 'pyemr': ['.git/*',
           '.git/hooks/*',
           '.git/info/*',
           '.git/logs/*',
           '.git/logs/refs/heads/*',
           '.git/logs/refs/remotes/origin/*',
           '.git/objects/00/*',
           '.git/objects/01/*',
           '.git/objects/02/*',
           '.git/objects/03/*',
           '.git/objects/06/*',
           '.git/objects/07/*',
           '.git/objects/08/*',
           '.git/objects/09/*',
           '.git/objects/0d/*',
           '.git/objects/0e/*',
           '.git/objects/0f/*',
           '.git/objects/11/*',
           '.git/objects/12/*',
           '.git/objects/15/*',
           '.git/objects/17/*',
           '.git/objects/1a/*',
           '.git/objects/1c/*',
           '.git/objects/1d/*',
           '.git/objects/1f/*',
           '.git/objects/20/*',
           '.git/objects/21/*',
           '.git/objects/23/*',
           '.git/objects/24/*',
           '.git/objects/25/*',
           '.git/objects/26/*',
           '.git/objects/27/*',
           '.git/objects/28/*',
           '.git/objects/29/*',
           '.git/objects/2a/*',
           '.git/objects/2b/*',
           '.git/objects/2c/*',
           '.git/objects/2d/*',
           '.git/objects/30/*',
           '.git/objects/31/*',
           '.git/objects/33/*',
           '.git/objects/35/*',
           '.git/objects/36/*',
           '.git/objects/38/*',
           '.git/objects/3b/*',
           '.git/objects/3d/*',
           '.git/objects/3e/*',
           '.git/objects/3f/*',
           '.git/objects/40/*',
           '.git/objects/41/*',
           '.git/objects/42/*',
           '.git/objects/44/*',
           '.git/objects/46/*',
           '.git/objects/47/*',
           '.git/objects/48/*',
           '.git/objects/4b/*',
           '.git/objects/4c/*',
           '.git/objects/4d/*',
           '.git/objects/4e/*',
           '.git/objects/4f/*',
           '.git/objects/50/*',
           '.git/objects/51/*',
           '.git/objects/52/*',
           '.git/objects/54/*',
           '.git/objects/56/*',
           '.git/objects/57/*',
           '.git/objects/58/*',
           '.git/objects/59/*',
           '.git/objects/5b/*',
           '.git/objects/5c/*',
           '.git/objects/5e/*',
           '.git/objects/5f/*',
           '.git/objects/61/*',
           '.git/objects/62/*',
           '.git/objects/63/*',
           '.git/objects/64/*',
           '.git/objects/67/*',
           '.git/objects/68/*',
           '.git/objects/69/*',
           '.git/objects/6a/*',
           '.git/objects/6b/*',
           '.git/objects/6d/*',
           '.git/objects/6e/*',
           '.git/objects/70/*',
           '.git/objects/71/*',
           '.git/objects/72/*',
           '.git/objects/73/*',
           '.git/objects/75/*',
           '.git/objects/77/*',
           '.git/objects/78/*',
           '.git/objects/7a/*',
           '.git/objects/7d/*',
           '.git/objects/80/*',
           '.git/objects/81/*',
           '.git/objects/82/*',
           '.git/objects/84/*',
           '.git/objects/85/*',
           '.git/objects/86/*',
           '.git/objects/88/*',
           '.git/objects/89/*',
           '.git/objects/8a/*',
           '.git/objects/8b/*',
           '.git/objects/8c/*',
           '.git/objects/8d/*',
           '.git/objects/8f/*',
           '.git/objects/90/*',
           '.git/objects/91/*',
           '.git/objects/93/*',
           '.git/objects/95/*',
           '.git/objects/96/*',
           '.git/objects/97/*',
           '.git/objects/98/*',
           '.git/objects/99/*',
           '.git/objects/9b/*',
           '.git/objects/9d/*',
           '.git/objects/9e/*',
           '.git/objects/a0/*',
           '.git/objects/a1/*',
           '.git/objects/a2/*',
           '.git/objects/a4/*',
           '.git/objects/a5/*',
           '.git/objects/a6/*',
           '.git/objects/a7/*',
           '.git/objects/a8/*',
           '.git/objects/a9/*',
           '.git/objects/aa/*',
           '.git/objects/ab/*',
           '.git/objects/ac/*',
           '.git/objects/ad/*',
           '.git/objects/af/*',
           '.git/objects/b0/*',
           '.git/objects/b1/*',
           '.git/objects/b3/*',
           '.git/objects/b5/*',
           '.git/objects/b6/*',
           '.git/objects/b8/*',
           '.git/objects/ba/*',
           '.git/objects/bc/*',
           '.git/objects/bd/*',
           '.git/objects/be/*',
           '.git/objects/bf/*',
           '.git/objects/c0/*',
           '.git/objects/c1/*',
           '.git/objects/c2/*',
           '.git/objects/c3/*',
           '.git/objects/c4/*',
           '.git/objects/c5/*',
           '.git/objects/c6/*',
           '.git/objects/c7/*',
           '.git/objects/c9/*',
           '.git/objects/ca/*',
           '.git/objects/cb/*',
           '.git/objects/cc/*',
           '.git/objects/cd/*',
           '.git/objects/ce/*',
           '.git/objects/cf/*',
           '.git/objects/d0/*',
           '.git/objects/d1/*',
           '.git/objects/d2/*',
           '.git/objects/d3/*',
           '.git/objects/d4/*',
           '.git/objects/d6/*',
           '.git/objects/d7/*',
           '.git/objects/d8/*',
           '.git/objects/d9/*',
           '.git/objects/da/*',
           '.git/objects/db/*',
           '.git/objects/dc/*',
           '.git/objects/de/*',
           '.git/objects/df/*',
           '.git/objects/e0/*',
           '.git/objects/e1/*',
           '.git/objects/e3/*',
           '.git/objects/e4/*',
           '.git/objects/e6/*',
           '.git/objects/e7/*',
           '.git/objects/e9/*',
           '.git/objects/eb/*',
           '.git/objects/ec/*',
           '.git/objects/ed/*',
           '.git/objects/ee/*',
           '.git/objects/f0/*',
           '.git/objects/f1/*',
           '.git/objects/f2/*',
           '.git/objects/f4/*',
           '.git/objects/f6/*',
           '.git/objects/f7/*',
           '.git/objects/f8/*',
           '.git/objects/f9/*',
           '.git/objects/fa/*',
           '.git/objects/fc/*',
           '.git/objects/fd/*',
           '.git/objects/fe/*',
           '.git/refs/heads/*',
           '.git/refs/remotes/origin/*',
           '.media/*'],
 'pyemr.pyemr': ['docker/*']}

install_requires = \
['awswrangler>=2.13.0,<3.0.0',
 'boto3>=1.20.23,<2.0.0',
 'fire>=0.4.0,<0.5.0',
 'loguru>=0.5.3,<0.6.0',
 'pandas>=1.3.5,<2.0.0',
 'poetry>=1.1.12,<2.0.0',
 'sh>=1.14.2,<2.0.0',
 'tomlkit==0.7.2',
 'tqdm>=4.62.3,<5.0.0']

entry_points = \
{'console_scripts': ['pyemr = pyemr.cli:main']}

setup_kwargs = {
    'name': 'pyemr',
    'version': '0.1.4',
    'description': 'A lightweight package for running poetry projects on emr.',
    'long_description': None,
    'author': 'Richard Brooker',
    'author_email': 'richjbrooker@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<3.10',
}


setup(**setup_kwargs)



"""A collection of aws tools"""
import os

import fire


def format_code():
    """ """
    os.system("pyment --output=google --write .")
    os.system(
        "autoflake --in-place --remove-unused-variables --remove-all-unused-imports **/*.py"
    )
    os.system("isort .")
    os.system("black .")


if __name__ == "__main__":
    fire.Fire(format_code)
